@@ -22,10 +22,10 @@ setClass("msSurv",representation(tree="graphNEL",
                                  cov.AJs="AorN",
                                  var.sop="AorN",
 				 cov.dA="AorN",
-                                 Fnorm="array",
-                                 Fsub="array",
-                                 Gnorm="array",
-                                 Gsub="array",
+                                 Fnorm="AorN",
+                                 Fsub="AorN",
+                                 Gnorm="AorN",
+                                 Gsub="AorN",
 				 Fnorm.var="AorN",
 				 Fsub.var="AorN",
 				 Gnorm.var="AorN",
